package com.tidecloud.dataacceptance.entity;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.BiConsumer;
import java.util.stream.Collectors;

/**   
 * @author: chudk 
 * @date:   2017年11月8日 上午11:27:47   
 */
public class SessionManager {

    private static volatile SessionManager instance = null;
    
    //key : sessionId **** value : Session
    private Map<String, Session> sessionIdMap;
    //key :  client deviceId **** value : sessionId
    private Map<String, String> deviceSessionMap;
    
    public static SessionManager getInstance(){
        if (instance == null) {
            synchronized (SessionManager.class) {
                if (instance == null) {
                    instance = new SessionManager();
                }
            }
        }
        return instance;
    }
    
    public SessionManager(){
        this.sessionIdMap = new ConcurrentHashMap<>();
        this.deviceSessionMap = new ConcurrentHashMap<>();
    }
    
    public boolean containsKey(String sessionId) {
        return sessionIdMap.containsKey(sessionId);
    }

    public boolean containsSession(Session session) {
        return sessionIdMap.containsValue(session);
    }

    public Session findBySessionId(String id) {
        return sessionIdMap.get(id);
    }

    public Session findByTerminalDevice(String phone) {
        String sessionId = this.deviceSessionMap.get(phone);
        if (sessionId == null){
            return null;
        }
        return this.findBySessionId(sessionId);
    }

    public synchronized Session put(String key, Session value) {
        if (value.getDeviceId() != null && !"".equals(value.getDeviceId().trim())) {
            this.deviceSessionMap.put(value.getDeviceId(), value.getId());
        }
        return sessionIdMap.put(key, value);
    }

    public synchronized Session removeBySessionId(String sessionId) {
        if (sessionId == null){
            return null;
        }
        Session session = sessionIdMap.remove(sessionId);
        if (session == null){
            return null;
        }
        if (session.getDeviceId() != null){
            this.deviceSessionMap.remove(session.getDeviceId());
        }
        return session;
    }

    // public synchronized void remove(String sessionId) {
    // if (sessionId == null)
    // return;
    // Session session = sessionIdMap.remove(sessionId);
    // if (session == null)
    // return;
    // if (session.getTerminalPhone() != null)
    // this.phoneMap.remove(session.getTerminalPhone());
    // try {
    // if (session.getChannel() != null) {
    // if (session.getChannel().isActive() || session.getChannel().isOpen()) {
    // session.getChannel().close();
    // }
    // session = null;
    // }
    // } catch (Exception e) {
    // e.printStackTrace();
    // }
    // }

    public Set<String> keySet() {
        return sessionIdMap.keySet();
    }

    public void forEach(BiConsumer<? super String, ? super Session> action) {
        sessionIdMap.forEach(action);
    }

    public Set<Entry<String, Session>> entrySet() {
        return sessionIdMap.entrySet();
    }

    public List<Session> toList() {
        return this.sessionIdMap.entrySet().stream().map(e -> e.getValue()).collect(Collectors.toList());
    }
    
}

